{------------------------------------------------------------------------------}
{
   Unit Name: astia
   Purpose  : Astian toteutus
   Author   : Vesa Lappalainen
   Date     : 6.12.2000
   Changed  :

   Astia on luokka, johon voidaan laittaa nestett maksimissaan
   Tilavuuden verran.
   Astiaan voidaan kaataa nestett toiseta astiasta.
   Astian kokonaan tyhjenemisest tai tyttymisest saadaan tapahtuma.
   Astian sisusta piirretn aina sinisell vrill suhteessa astian
   korkeuteen.  Eli tysi astia on kokonaan sininen ja tyhj kokonaan
   valkoinen.

}
{------------------------------------------------------------------------------}


unit astia;

interface
uses extctrls,Graphics;

type
  TAstia = class;
  TAstiaEvent = procedure (sender:TAstia; const viesti:string) of object;

  TAstia = class(TPaintBox)
  private
    FTilavuus: double;
    FAinetta: double;
    viesti : string;
    FOnFull: TAstiaEvent;
    FOnEmpty: TAstiaEvent;
    procedure SetTilavuus(const Value: double);
    function GetMahtuu: double;
    procedure SetAinetta(const Value: double);
    procedure SetMahtuu(const Value: double);
    procedure CheckEvents;
  public
    procedure Paint; override;
    function Kaada(a:TAstia) : double;
  published
    property Tilavuus : double read FTilavuus write SetTilavuus;
    property Ainetta : double read FAinetta write SetAinetta;
    property Mahtuu : double read GetMahtuu write SetMahtuu;
    property OnFull : TAstiaEvent read FOnFull write FOnFull;
    property OnEmpty : TAstiaEvent read FOnEmpty write FOnEmpty;
  end;

  function eq(d1,d2:double):boolean;

implementation

const EPS = 0.001;

function eq(d1,d2:double):boolean;
// Verrataan onko kaksi reaalilukua yhtsuuria EPS:in tarkkuudella
begin
  Result := false;
  if ( d1 < d2 - EPS ) then exit;
  if ( d1 > d2 + EPS ) then exit;
  Result := true;
end;

{ TAstia }

function TAstia.GetMahtuu: double;
begin
  Result := Tilavuus - Ainetta;
end;

procedure TAstia.Paint;
begin
  inherited;
  Canvas.Brush.Color := clWhite;
  Canvas.Rectangle(0,0,Width,Height);
  if ( eq(Tilavuus,0) ) then exit;
  Canvas.Brush.Color := clBlue;
  Canvas.Rectangle(0,Round(Height*Mahtuu/Tilavuus),Width,Height);
end;

procedure TAstia.CheckEvents;
begin
  if ( eq(Ainetta,0) and Assigned(OnEmpty) ) then OnEmpty(self,viesti);
  if ( eq(Ainetta,Tilavuus) and Assigned(OnFull) ) then OnFull(self,viesti);
end;

procedure TAstia.SetAinetta(const Value: double);
begin
  if ( eq(Ainetta,Value) ) then exit;
  FAinetta := Value;
  invalidate;
  CheckEvents;
  viesti := '';
end;

procedure TAstia.SetMahtuu(const Value: double);
begin
  Ainetta := Tilavuus - Value;
  invalidate;
end;

procedure TAstia.SetTilavuus(const Value: double);
begin
  FTilavuus := Value;
  invalidate;
end;

function TAstia.Kaada(a: TAstia): double;
// Kaataa astiaan (self) nestett toisesta astiasta (a).
// Jos kaikki a:n neste mahtuu, niin se kytetn ja a menee tyhjksi.
// Jos kaikki ei mahdu, niin laitetaan sen verran kuin mahtuu
// eli self tulee tyteen ja a:sta vhenee neste.
// Palautetaan a:han jljelle jneen nesteen mr.
// Sijoituksissa pit olla tarkkana, jotta tapahtuman (tyhj/tysi)
// tullessa tilanne on kummankin astia kohdalta oikein.
// Kummankin astian apuattribuuttiin viesti laitetaan merkkijonona
// tieto mist astiasta ollaan kaatamassa ja mihin (esim. a5 -> a8 ).
begin
  Result := a.Ainetta - Mahtuu;
  viesti := a.Name + ' -> ' + name;
  a.viesti := viesti;
  if ( Result <= 0 ) then begin
    Result := 0;
    FAinetta := Ainetta + a.Ainetta;
    a.Ainetta := 0;
    invalidate;
    CheckEvents;
    exit;
  end;

  a.Ainetta := Result;
  Mahtuu := 0;
end;

end.
